module Shotengai
  module Controller
    module Merchant
      class ProductSnapshotsController < Shotengai::Controller::Merchant::Base
        self.base_resources = ProductSnapshot
        self.template_dir = 'shotengai/merchant/snapshots/'
        
        remove_actions :create, :destroy

        def default_query resources
          resources.in_order  
        end

        def index_query resources
          resources.where(
              params[:order_id] && { shotengai_order_id: params[:order_id] }
            ).where(
              params[:product_series_id] && { shotengai_series_id: params[:product_series_id] }
            )
        end

        def update
          @resource.class.transaction do
            @resource.attributes = resource_params
            @resource.has_changes_to_save? && edit_only_unpaid && @resource.save!
            @resource.update_columns(rewrite_params.to_h) if rewrite_params.present? # skip validation
          end
          head 200
        end
        
        private
          def resource_params
            remark_value = params.require(resource_key).fetch(:remark_value, nil)&.permit!
            params.require(resource_key).permit(
              :revised_amount
              # WARNING: if .fetch(:remark_value, {}) get value {}
              #   it would update to this value
            ).merge(remark_value: remark_value || @resource.remark_value)
          end

          def rewrite_params
            params.require(resource_key).permit(spec_rewrite: {})
          end

          def edit_only_unpaid
            @resource.order.unpaid? || raise(Shotengai::WebError.new('订单已支付，不可修改该快照。', '-1', 403))
          end

          def manager_auth
            @manager = params[:manager_type].constantize.find(params[:manager_id])
          end
      end
    end
  end
end
